/*
*****************************************************************************
* Copyright (C) 2019 Infineon Technologies AG. All rights reserved.
*
* Infineon Technologies AG (INFINEON) is supplying this file for use
* exclusively with Infineon's products. This file can be freely
* distributed within development tools and software supporting such microcontroller
* products.
*
* THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
* INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR DIRECT, INDIRECT, INCIDENTAL,
* ASPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*
******************************************************************************
*/

#include "src/xmc1100/i2c_int/i2c_int.h"

#include "xmc_i2c.h"
#include "XMC1100.h"
#include "src/xmc1100/interrupt/interrupts.h"
#include "src/xmc1100/time/time.h"
#include "src/xmc1100/atomic/atomic.h"
#include "src/xmc1100/i2c/conf_i2c.h"


// reset address
#define TLE493D_AW2B6_I2C_RESET_ADDR	0x00U
// recovery address
#define TLE493D_AW2B6_I2C_RECOV_ADDR	0xFFU


static struct {
	volatile const uint8_t* data_write;
	volatile uint8_t* data_read;
	volatile uint8_t index, count;
	volatile uint8_t read_operation;
	volatile uint8_t running;
	I2C_INT_state_t *handler_state;
} _local;


static inline void _protect_on(void);
static inline void _protect_off(void);


// send stop condition and end frame
static inline void _stop(I2C_INT_state_t state)
{
	// send stop condition
	if (I2C_INT_NACK != state) {
		XMC_I2C_CH_MasterStop(XMC_I2C0_CH1);
	}

	// wait for stop
	while (XMC_USIC_CH_TBUF_STATUS_BUSY == XMC_USIC_CH_GetTransmitBufferStatus(XMC_I2C0_CH1))
		; // wait

	// activate protection
	_protect_on();

	// notify GUI of new data
	if (_local.read_operation) {
		// TODO: new data frame received
	}

	// set I2C status
	if (NULL != (_local.handler_state)) {
		*_local.handler_state = state;
	}

	// mark peripheral as not being used
	_local.running = false;
}


// SR0: Standard Receive, Alternative Receive
void USIC0_0_IRQHandler ()
{
	// copy received data in buffer
	_local.data_read[_local.index ++] = XMC_I2C_CH_GetReceivedData(XMC_I2C0_CH1);

	// all data received
	if (_local.index == _local.count) {
		_stop(I2C_INT_SUCCESS);
	}
	// one more byte needed
	else if (_local.index == (_local.count - 1)) {
		// prepare to read one more byte and send NACK
		XMC_I2C_CH_MasterReceiveNack(XMC_USIC0_CH1);
	}
	// more then one more byte expected
	else {
		// prepare to read one more byte and send ACK
		// (and request more data)
		XMC_I2C_CH_MasterReceiveAck(XMC_USIC0_CH1);
	}
}

// ACK, NACK, ERROR
void USIC0_2_IRQHandler ()
{
	// master received ACK from slave
	if (XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED & XMC_I2C0_CH1->PSR) {
		// master reading
		if (_local.read_operation) {
			// accept data byte and request more
			if (_local.count >= 2) {
				XMC_I2C_CH_MasterReceiveAck(XMC_USIC0_CH1);
			}
			// accept data byte, do not request more
			else if (_local.count == 1) {
				XMC_I2C_CH_MasterReceiveNack(XMC_USIC0_CH1);
			}
			// master reading 0 bytes of data
			else {
				_stop(I2C_INT_SUCCESS);
			}
		}
		// master writing
		else {
			// write byte of data
			if (_local.index < _local.count) {
				XMC_I2C_CH_MasterTransmit(XMC_I2C0_CH1, _local.data_write[_local.index++]);
			}
			// writing complete
			else {
				_stop(I2C_INT_SUCCESS);
			}
		}
	}
	// master received NACK from slave
	else if (XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED & XMC_I2C0_CH1->PSR) {
		// no data sent to device
		// so this NACK is from the device addressing
		if (0 == _local.index) {
			_stop(I2C_INT_NACK);
		}
		// device was addressed successfully
		// (it is possible that not all data was accepted by device,
		// in Master Write mode)
		else {
			_stop(I2C_INT_SUCCESS);
		}
	}
	// error condition
	else {
		// TODO: error condition
	}

	// clear flags: ACK, NACK
	XMC_I2C_CH_ClearStatusFlag(
		XMC_I2C0_CH1,
		XMC_I2C_CH_STATUS_FLAG_ACK_RECEIVED
		| XMC_I2C_CH_STATUS_FLAG_NACK_RECEIVED
	);
}


void I2C_INT_init(void)
{
	// init I2C Channel
	XMC_I2C_CH_Init(XMC_USIC0_CH1, &I2C_master_conf);

	/* Configure GPIO, P2 also needs to be set to digital - mode set is not enough */
	XMC_GPIO_Init(CONF_I2C_SDA_PIN, &CONF_I2C_SDA_PIN_DISABLED);
	XMC_GPIO_Init(CONF_I2C_SCL_PIN, &CONF_I2C_SCL_PIN_DISABLED);

	// configure TLx port for I2C output
	WR_REG(PORT2->IOCR8, PORT2_IOCR8_PC11_Msk, PORT2_IOCR8_PC11_Pos, 0x1EU);
	WR_REG(PORT2->IOCR8, PORT2_IOCR8_PC10_Msk, PORT2_IOCR8_PC10_Pos, 0x1FU);

	// set SCL to always read logical 1
	// instead of pin value.
	// Protection automatically disabled during communication.
	// Required to prevent a pulse on SCL
	// to cause I2C peripheral to freeze.
	_protect_on();

	// { interrupts }
	// on word received (for Read mode)
	// and (N)ACK (for master start and write mode)
	XMC_I2C_CH_EnableEvent(
			XMC_USIC0_CH1,
			XMC_I2C_CH_EVENT_STANDARD_RECEIVE
			| XMC_I2C_CH_EVENT_ALTERNATIVE_RECEIVE
			| XMC_I2C_CH_EVENT_ACK
			| XMC_I2C_CH_EVENT_NACK
			| XMC_I2C_CH_EVENT_ERROR
	);

	// { route interrupts }
	// interrupt: Receive (std and alt) to SR0
	XMC_I2C_CH_SelectInterruptNodePointer(XMC_USIC0_CH1, XMC_USIC_CH_INTERRUPT_NODE_POINTER_RECEIVE, 0);
	XMC_I2C_CH_SelectInterruptNodePointer(XMC_USIC0_CH1, XMC_USIC_CH_INTERRUPT_NODE_POINTER_ALTERNATE_RECEIVE, 0);
	// interrupt: protocol specific (for ACK, NACK, ERROR) to SR2
	XMC_I2C_CH_SelectInterruptNodePointer(XMC_USIC0_CH1, XMC_USIC_CH_INTERRUPT_NODE_POINTER_PROTOCOL, 2);

	// enable NVIC interrupts for SR0,SR1,SR2
	NVIC_ClearPendingIRQ(USIC0_0_IRQn);
	NVIC_ClearPendingIRQ(USIC0_2_IRQn);
	NVIC_EnableIRQ(USIC0_0_IRQn);
	NVIC_EnableIRQ(USIC0_2_IRQn);

	// set USIC channel mode to I2C (enable I2C)
	XMC_I2C_CH_Start(XMC_USIC0_CH1);
}


__attribute__((always_inline))
static inline bool _start(uint8_t addr, uint8_t* data, uint8_t count, I2C_INT_state_t* handler, uint8_t read)
{
	// enter atomic block
	atomic_enter();

	// check if another I2C operation running
	if (_local.running) {
		atomic_exit();
		return false;
	}

	// clear all status flags (first 16 bits)
	XMC_I2C0_CH1->PSCR = (1u << 16) -1;

	// disable I2C protection
	_protect_off();

	// start I2C communication
	//XMC_I2C_CH_MasterStart(XMC_I2C0_CH1, addr, (read ? XMC_I2C_CH_CMD_READ : XMC_I2C_CH_CMD_WRITE));
	XMC_I2C0_CH1->TBUF[0] = addr | ((uint32_t)4U << 8U) | (read & 1u);


	// wait for device, check if peripheral frozen
	int counter = 20;
	while (XMC_USIC_CH_TBUF_STATUS_BUSY == XMC_USIC_CH_GetTransmitBufferStatus(XMC_I2C0_CH1)) {
		wait(5);
		counter --;
		if (counter <= 0) {
			NVIC_SystemReset();
		}
	}


	// set I2C as running
	_local.running = 1;

	// copy buffer information locally
	_local.read_operation = read;
	if (0 != read) {
		_local.data_read = data;
	}
	else {
		_local.data_write = (const uint8_t *)data;
	}
	_local.count = count;
	_local.index = 0;
	_local.handler_state = handler;
	if (NULL != handler) {
		*(handler) = I2C_INT_IN_PROGRESS;
	}

	// exit atomic block
	atomic_exit();

	return true;
}


bool I2C_INT_write(uint8_t addr, const uint8_t* data, uint8_t count, I2C_INT_state_t* handle)
{
	return _start(addr, (uint8_t *)data, count, handle, 0);
}


bool I2C_INT_read(uint8_t addr, uint8_t *data, uint8_t count, I2C_INT_state_t* handle)
{
	return _start(addr, data, count, handle, 1);
}


__attribute__((always_inline))
static inline void _protect_on()
{
	// always read logical 1 on SDA
	XMC_USIC0_CH1->DXCR[XMC_I2C_CH_INPUT_SCL] |= USIC_CH_DXCR_DSEL_Msk;
	XMC_USIC0_CH1->DXCR[XMC_I2C_CH_INPUT_SDA] |= USIC_CH_DXCR_DSEL_Msk;

	// configure pins for GPIO input
	WR_REG(PORT2->IOCR8, PORT2_IOCR8_PC11_Msk, PORT2_IOCR8_PC11_Pos, 0x0U);
	WR_REG(PORT2->IOCR8, PORT2_IOCR8_PC10_Msk, PORT2_IOCR8_PC10_Pos, 0x0U);

	// enable external interrupts
	INT_int_ext_enable();
}


__attribute__((always_inline))
static inline void _protect_off()
{
	// disable external interrupts
	INT_int_ext_disable();

	// configure TLx port for I2C output
	WR_REG(PORT2->IOCR8, PORT2_IOCR8_PC11_Msk, PORT2_IOCR8_PC11_Pos, 0x1EU);
	WR_REG(PORT2->IOCR8, PORT2_IOCR8_PC10_Msk, PORT2_IOCR8_PC10_Pos, 0x1FU);

	// SDA will read value from pin P2_10
	XMC_USIC0_CH1->DXCR[XMC_I2C_CH_INPUT_SDA] = (XMC_USIC0_CH1->DXCR[XMC_I2C_CH_INPUT_SDA] & ~USIC_CH_DXCR_DSEL_Msk)
												| (USIC0_C1_DX0_P2_10 << USIC_CH_DXCR_DSEL_Pos);
	// SCL will read value from pin P2_11
	XMC_USIC0_CH1->DXCR[XMC_I2C_CH_INPUT_SCL] = (XMC_USIC0_CH1->DXCR[XMC_I2C_CH_INPUT_SCL] & ~USIC_CH_DXCR_DSEL_Msk)
												| (USIC0_C1_DX1_P2_11 << USIC_CH_DXCR_DSEL_Pos);
}


void I2C_INT_write_reset(void)
{
	NVIC_DisableIRQ(USIC0_0_IRQn);
	NVIC_DisableIRQ(USIC0_2_IRQn);

	_protect_off();

	XMC_I2C_CH_MasterStart(
		XMC_USIC0_CH1,
		TLE493D_AW2B6_I2C_RESET_ADDR,
		XMC_I2C_CH_CMD_WRITE
	);
	wait(10);
	XMC_I2C_CH_MasterStop(XMC_USIC0_CH1);

	// wait for stop
	int count = 0;
	while (XMC_USIC_CH_TBUF_STATUS_BUSY == XMC_USIC_CH_GetTransmitBufferStatus(XMC_I2C0_CH1)) {
		if (++ count > 100000) {
			break;
		}
	}

	_protect_on();

	NVIC_ClearPendingIRQ(USIC0_0_IRQn);
	NVIC_ClearPendingIRQ(USIC0_2_IRQn);
	NVIC_EnableIRQ(USIC0_0_IRQn);
	NVIC_EnableIRQ(USIC0_2_IRQn);
}


void I2C_INT_write_recover(void)
{
	NVIC_DisableIRQ(USIC0_0_IRQn);
	NVIC_DisableIRQ(USIC0_2_IRQn);

	_protect_off();

	XMC_I2C_CH_MasterStart(
		XMC_USIC0_CH1,
		TLE493D_AW2B6_I2C_RECOV_ADDR,
		XMC_I2C_CH_CMD_READ
	);
	wait(10);

	XMC_I2C_CH_MasterStop(XMC_USIC0_CH1);

	// wait for stop
	int count = 0;
	while (XMC_USIC_CH_TBUF_STATUS_BUSY == XMC_USIC_CH_GetTransmitBufferStatus(XMC_I2C0_CH1)) {
		if (++ count > 100000) {
			break;
		}
	}

	_protect_on();

	NVIC_ClearPendingIRQ(USIC0_0_IRQn);
	NVIC_ClearPendingIRQ(USIC0_2_IRQn);
	NVIC_EnableIRQ(USIC0_0_IRQn);
	NVIC_EnableIRQ(USIC0_2_IRQn);
}
