/*
*****************************************************************************
* Copyright (C) 2019 Infineon Technologies AG. All rights reserved.
*
* Infineon Technologies AG (INFINEON) is supplying this file for use
* exclusively with Infineon's products. This file can be freely
* distributed within development tools and software supporting such microcontroller
* products.
*
* THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
* INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR DIRECT, INDIRECT, INCIDENTAL,
* ASPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*
******************************************************************************
*/


/*! \file i2c_int.h
 * \brief Interrupt-based I2C Driver for XMC1100 USIC Module */


#ifndef SRC_XMC1100_I2C_INT_I2C_INT_H_
#define SRC_XMC1100_I2C_INT_I2C_INT_H_


#include <stdint.h>
#include <stdbool.h>


//! Status values returned by the I2C read/write commands
typedef volatile enum {
	I2C_INT_SUCCESS,
	I2C_INT_IN_PROGRESS,
	I2C_INT_ERR_PROTOCOL,
	I2C_INT_NACK,
} I2C_INT_state_t;


//! Initialize the I2C interrupt-based peripheral
void I2C_INT_init(void);


//! start asynchronous I2C write
bool I2C_INT_write(uint8_t addr, const uint8_t* data, uint8_t count, I2C_INT_state_t* handle);

//! start asynchronous I2C read
bool I2C_INT_read(uint8_t addr, uint8_t *data, uint8_t count, I2C_INT_state_t* handle);

//! start blocking I2C write
static inline I2C_INT_state_t I2C_INT_write_block(uint8_t addr, const uint8_t* data, uint8_t count)
{
	volatile I2C_INT_state_t state;

	// attempt to start I2C transfer until successful
	while (!I2C_INT_write(addr, data, count, &state))
		;
	while (I2C_INT_IN_PROGRESS == state)
		; //wait

	return state;
}

//! start blocking I2C read
static inline I2C_INT_state_t I2C_INT_read_block(uint8_t addr, uint8_t *data, uint8_t count)
{
	volatile I2C_INT_state_t state;

	// attempt to start I2C transfer until successful
	while (!I2C_INT_read(addr, data, count, &state))
		;
	while (I2C_INT_IN_PROGRESS == state)
		; //wait

	return state;
}


//! Write Reset address (for TLx493D sensors) on I2C bus (blocking)
void I2C_INT_write_reset(void);


//! Write Recover address (for TLx493D sensors) on I2C bus (blocking)
void I2C_INT_write_recover(void);


#endif /* SRC_XMC1100_I2C_INT_I2C_INT_H_ */
